package laboratoire4;

import java.util.ArrayList;
import java.util.List;

public class PusherBoard {
    private final Player player;
    private Pawn[][] board;

    private final List<Pawn> maxPawns = new ArrayList<>();
    private final List<Pawn> minPawns = new ArrayList<>();

    public PusherBoard(Player player, String[] boardValues) {
        this.player = player;

        newGame(boardValues);
    }

    public void newGame(String[] boardValues) {
        this.board = new Pawn[8][8];

        int col = 0, row = 0;
        for (String boardValue : boardValues) {
            int v = Integer.parseInt(boardValue);
            if (v != 0) {
                Player pawnPlayer = (v == 1 || v == 2) ? Player.RED : Player.BLACK;
                Pawn pawn;
                if (v % 2 == 0) {    // 2 et 4 sont les pushers
                    pawn = new Pusher(pawnPlayer, row, col);
                } else {
                    pawn = new Pushed(pawnPlayer, row, col);
                }

                if (pawnPlayer == player) {
                    maxPawns.add(pawn);
                } else {
                    minPawns.add(pawn);
                }

                board[row][col] = pawn;
            }

            col++;
            if (col == board.length) {
                col = 0;
                row++;
            }
        }
    }

    public String runNextMove() {
        MiniMax.MiniMaxResult result = MiniMax.miniMax(this);
        Pawn pawn = result.getPawn();
        String initialPosition = pawn.getPosition();

        move(pawn, result.getMovement());

        return initialPosition + "-" + pawn.getPosition();
    }

    public void move(String move) {
        //FORMAT ex : D2-D3
        String[] split = move.trim().split(" - ");
        move(split[0], split[1]);
    }

    private void move(Pawn pawn, Pawn.PawnMovement movement) {
        move(pawn.getCol(), pawn.getRow(), pawn.getCol() + movement.getMove(), pawn.getRow() + pawn.getDirection());
    }

    public void move(String from, String to) {
        //FORMAT ex : from {D2}, to {D3}
        int from_col = (int) from.charAt(0) - 65;
        int from_row = Integer.parseInt(String.valueOf(from.charAt(1))) - 1;
        int to_col = (int) to.charAt(0) - 65;
        int to_row = Integer.parseInt(String.valueOf(to.charAt(1))) - 1;

        move(from_col, from_row, to_col, to_row);
    }

    public void move(int from_col, int from_row, int to_col, int to_row) {
        if (!isValid(from_col, from_row, to_col)) {
            return;
        }

        Pawn pawn = board[from_row][from_col];
        Pawn destPawn = board[to_row][to_col];

        if (destPawn != null) {
            if (destPawn.getPlayer() == player) {
                maxPawns.remove(destPawn);
            } else {
                minPawns.remove(destPawn);
            }
        }

        board[from_row][from_col] = null;
        board[to_row][to_col] = pawn;

        pawn.setRow(to_row);
        pawn.setCol(to_col);
    }


    private boolean isValid(int from_col, int from_row, int to_col) {
        Pawn pawn = getBoard()[from_row][from_col];
        Pawn.PawnMovement move = Pawn.PawnMovement.from(to_col - from_col);

        if (pawn == null) {
            return false;
        }

        return pawn.isMoveValid(this, move);
    }

    public Pawn[][] getBoard() {
        return board;
    }

    public List<Pawn> getMaxPawns() {
        return maxPawns;
    }

    public List<Pawn> getMinPawns() {
        return minPawns;
    }

    public void printBoard() {
        for (int i = 7; i >= 0; i--) {
            for (int j = 0; j < this.board.length; j++) {
                if (this.board[i][j] != null) {
                    System.out.print(this.board[i][j] + " | ");
                } else {
                    System.out.print("                   | ");
                }
            }
            System.out.println();
            System.out.println("----------------------------------------------------------------------------------------");
        }
    }
}
